@@ -29,6 +29,7 @@
 #define __JSON_GENERATOR_H__
 
 #include <json-glib/json-types.h>
+#include <gio/gio.h>
 
 G_BEGIN_DECLS
 
@@ -76,14 +77,30 @@ struct _JsonGeneratorClass
 
 GType json_generator_get_type (void) G_GNUC_CONST;
 
-JsonGenerator *json_generator_new (void);
-gchar *        json_generator_to_data  (JsonGenerator  *generator,
-                                        gsize          *length);
-gboolean       json_generator_to_file  (JsonGenerator  *generator,
-                                        const gchar    *filename,
-                                        GError        **error);
-void           json_generator_set_root (JsonGenerator  *generator,
-                                        JsonNode       *node);
+JsonGenerator * json_generator_new              (void);
+
+void            json_generator_set_pretty       (JsonGenerator  *generator,
+                                                 gboolean        is_pretty);
+gboolean        json_generator_get_pretty       (JsonGenerator  *generator);
+void            json_generator_set_indent       (JsonGenerator  *generator,
+                                                 guint           indent_level);
+guint           json_generator_get_indent       (JsonGenerator  *generator);
+void            json_generator_set_indent_char  (JsonGenerator  *generator,
+                                                 gunichar        indent_char);
+gunichar        json_generator_get_indent_char  (JsonGenerator  *generator);
+void            json_generator_set_root         (JsonGenerator  *generator,
+                                                 JsonNode       *node);
+JsonNode *      json_generator_get_root         (JsonGenerator  *generator);
+
+gchar *         json_generator_to_data          (JsonGenerator  *generator,
+                                                 gsize          *length);
+gboolean        json_generator_to_file          (JsonGenerator  *generator,
+                                                 const gchar    *filename,
+                                                 GError        **error);
+gboolean        json_generator_to_stream        (JsonGenerator  *generator,
+                                                 GOutputStream  *stream,
+                                                 GCancellable   *cancellable,
+                                                 GError        **error);
 
 G_END_DECLS
 
